package com.wzhi.demo

import zio._
import scala.language.postfixOps
import Money.tools._

object CheckBalance extends App {
  import zio.console._

  import Tools.MapTool._
  import sameShape._

  def calculateBalance[T <: Currency](transactions: List[Transaction[T]])
                                     (implicit zeroMoney: Money[T]): Map[AccountID, Money[T]] = {
    import TransType._
    val (inBoundTrans, outBoundTrans) = transactions.groupBy(_.accountId).partitionValues(_.direction == InBound)
    import cats.implicits._
    val inBoundMoney = inBoundTrans.mapValueAs(_.map(_.money).combineAll)
    val outBoundMoney = outBoundTrans.mapValueAs(_.map(_.money).combineAll)
    (inBoundMoney outerJoin outBoundMoney) calcValuesAs (_ - _)
  }

  def run(args: List[String]): ZIO[ZEnv, Nothing, ExitCode] = {
    implicit val zeroMoney: Money[USD.type] = 0 as USD
    val program = for {
      fiber1 <- ZIO.succeed(DataRepository.getUsdTransactions).fork
      fiber2 <- ZIO.succeed(DataRepository.getBalances).fork
      transactions <- fiber1.join
      balances <- fiber2.join
      calculatedBalances <- ZIO.succeed(calculateBalance(transactions))
      warnings <- ZIO.succeed(balances differences calculatedBalances)
    } yield {
      println(s"$balances")
    }

    putStrLn("Start Program") *> program *> ZIO.succeed(ExitCode(0))

  }
}
