package com.wzhi.demo

object FactoryShow extends App{
  trait Money2 {
    val amount: BigDecimal
  }
  case class UsdMoney(amount: BigDecimal) extends Money2
  case class CnyMoney(amount: BigDecimal) extends Money2

  object OOfactory {
    def createMoney(amount: BigDecimal, currency: Currency): Money2 = { // 类型擦除
      if (currency == USD) return UsdMoney(amount);
      else if (currency == CNY) return CnyMoney(amount);
      else return null;
    }
  }
  val usd = OOfactory.createMoney(23, USD)
  val cny = OOfactory.createMoney(23, CNY)
  val jpy = OOfactory.createMoney(23, JPY)
  println(usd.amount)
  println(jpy.amount)           // 运行时崩溃

  object FPFactory {
    case class CreateMoney[T <: Money2](fac: BigDecimal => T) {
      def create(amount: BigDecimal): T = fac(amount) // 返回值保留了它本身的类型
    }
    implicit class createUSD(currency: USD.type) extends CreateMoney(x => UsdMoney(x))
    implicit class createCNY(currency: CNY.type) extends CreateMoney(x => CnyMoney(x))
  }
  import FPFactory._
  val typedUSD = USD.create(23)
  val typedCNY = CNY.create(23)
  //val typedJPY = JPY.create(23)  // 编译器发现你没有定义过JPY的创建方法
  println(typedUSD)

  implicit class usdMoneyOps(money: UsdMoney) {
    def signShow: String = s"$$ $money.amount"
  }
  typedUSD.signShow
  //usd.signShow      // 工厂模式擦除了不同返回的子类型，cast它们去了父类型。使得编译器失去了对子类型的判断能力
}
