package com.wzhi.demo

import zio._

import scala.language.postfixOps

object FailedCheckBalance extends App {
  import zio.console._

  import Tools.MapTool._
  import sameShape._

  def calculateBalance[T <: Currency](transactions: List[Transaction[T]]): Map[AccountID, Money[USD.type]] = {
    import Money.tools._
    if (transactions.isEmpty) Map()
    implicit val zeroMoney = 0 as transactions.head.money.currency

    def sumTransMoney(transactions: Map[AccountID, List[Transaction[T]]]): Map[AccountID, Money[T]] = {
      import cats.implicits._
      transactions.map {
        case (accountId, transactions: List[Transaction[T]]) => (accountId, transactions.map(_.money).combineAll)
      }
    }

    import TransType._
    val (inBoundTrans, outBoundTrans) = transactions.groupBy(_.accountId).partitionValues(_.direction == InBound)
    val inBoundMoney = sumTransMoney(inBoundTrans)
    val outBoundMoney = sumTransMoney(outBoundTrans)
    (inBoundMoney outerJoin outBoundMoney).combineValuesOn(_ - _).mapValueAs(_.amount as USD)
  }

  def run(args: List[String]): ZIO[ZEnv, Nothing, ExitCode] = {
    val program = for {
      fiber1 <- ZIO.fail(new RuntimeException("Something Wrong")).fork
      fiber2 <- ZIO.succeed(DataRepository.getBalances).fork
      balances <- fiber2.join
      transactions <- fiber1.join
      calculatedBalances <- ZIO.succeed(calculateBalance(transactions))
      warnings <- ZIO.succeed(balances differences calculatedBalances)
    } yield {
      println(s"$warnings")
    }

    putStrLn("Start Program") *>
      program.retryN(3).catchAllCause(cause => putStrLn(s"${cause.prettyPrint}") as ExitCode(1)).
        provideLayer(Console.live).exitCode
  }
}
