package com.wzhi.demo

sealed trait Currency

case object USD extends Currency { override def toString: String = "United States Dollar"}
case object CNY extends Currency { override def toString: String = "China Yuan"}
case object JPY extends Currency { override def toString: String = "Japan Yuan"}

case class Money[C <: Currency] (amount: BigDecimal, currency: C)

case class AccountID(id: Long) {
  override def toString: String = s"ID: $id"
}

object Money {
  object tools {
    implicit class BigDecimalOps(m: BigDecimal) {
      def as[C <: Currency](curr: C): Money[C] = Money[C](m, curr)
    }
    implicit class DoubleMoneyOps(m: Double) {
      def as[C <: Currency](curr: C): Money[C] = Money[C](m, curr)
    }
    implicit class MoneyOps[C <: Currency](m: Money[C]) {
      def - (money: Money[C]): Money[C] = Money[C] (m.amount - money.amount, m.currency)
    }

    import cats.Monoid
    implicit def moneyMonoid[C <: Currency]: Monoid[Money[C]] = new Monoid[Money[C]] {
      def empty: Money[C] = new Money[C](0, null.asInstanceOf[C])
      def combine(x: Money[C], y: Money[C]): Money[C] =
        if (x.currency != null) (x.amount + y.amount) as x.currency
        else (x.amount + y.amount) as y.currency
    }
  }
}
