package com.wzhi.demo

object Tools {
  def partitionMapValues[A, B, F[B] <: Iterable[B]](map: Map[A, F[B]], condition: B => Boolean): (Map[A, F[B]], Map[A, F[B]]) = {
    val (map1: List[(A, F[B])], map2: List[(A, F[B])]) = map.map { case (key, values: F[B]) =>
      val (values1, values2) = values.partition(condition)
      ((key, values1), (key, values2))
    }.unzip
    (map1.filter(_._2.nonEmpty).toMap, map2.filter(_._2.nonEmpty).toMap)
  }

  def outerJoinSameShapeMap[A, B](leftMap: Map[A, B], rightMap: Map[A, B])(f: (B, B) => B, empty: B): Map[A, B] = {
    val leftJoin: Map[A, B] = leftMap.map {
      case (a, b) => rightMap.get(a) match {
        case Some(c) => (a, f(b, c))
        case None => (a, f(b, empty))
      }
    }
    val restJoin: Map[A, B] = rightMap.collect {
      case (a, b) if !leftMap.contains(a) => (a, f(empty, b))
    }
    leftJoin ++ restJoin
  }

  def findSameShapeMapDifferences[A, B](leftMap: Map[A, B], rightMap: Map[A, B]): Map[A, (Option[B], Option[B])] = {
    val leftJoin: Map[A, (Option[B], Option[B])] = leftMap.collect {
      case (key, value) if rightMap.get(key).isEmpty || rightMap.get(key) != Option(value) =>
        (key, (Option(value), rightMap.get(key)))
    }
    val restJoin = rightMap.collect {
      case (a, b) if !leftMap.contains(a) => (a, (None, Option(b)))
    }
    leftJoin ++ restJoin
  }

  object MapTool {
    implicit class MapPartition[KEY, B, F[B] <: Iterable[B]](map: Map[KEY, F[B]]) {
      def partitionValues(condition: B => Boolean): (Map[KEY, F[B]], Map[KEY, F[B]]) =
        Tools.partitionMapValues(map, condition)
    }

    object sameShape {
      implicit class MapTools[KEY, VALUE](thisMap: Map[KEY, VALUE]) {
        def outerJoin(rightMap: Map[KEY, VALUE])(implicit zeroValue: VALUE) = new {
          def combineValuesOn(joinValue: (VALUE, VALUE) => VALUE): Map[KEY, VALUE] =
            Tools.outerJoinSameShapeMap(thisMap, rightMap)(joinValue, zeroValue)

          val calcValuesAs = combineValuesOn _
        }
        def differences(rightMap: Map[KEY, VALUE]): Map[KEY, (Option[VALUE], Option[VALUE])] = {
          findSameShapeMapDifferences(thisMap, rightMap)
        }
        def mapValueAs[MAP_TO](f: VALUE => MAP_TO): Map[KEY, MAP_TO] = thisMap.map{
          case (key, value) => (key, f(value))
        }
      }
    }
  }


}
