package com.wzhi.demo.java;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class JavaCode {
    public Map<AccountId, List<Transcation>> groupByAccountID(List<Transcation> transactionList) {
        Map<AccountId, List<Transcation>> result = new HashMap<>();
        for(Transcation transaction : transactionList) {
            AccountId id = transaction.getAccountId();
            if(result.containsKey(id)) {
                result.get(id).add(transaction);
            }
            else {
                List<Transcation> newValueList = new LinkedList<>();
                newValueList.add(transaction);
                result.put(id, newValueList);
            }
        }
        return result;
    }
    public List<Map<AccountId, List<Transcation>>> partitionInOut(Map<AccountId, List<Transcation>> accountTransMap) {
        Map<AccountId, List<Transcation>> part1 = new HashMap<>();
        Map<AccountId, List<Transcation>> part2 = new HashMap<>();
        for(Map.Entry<AccountId, List<Transcation>> entry: accountTransMap.entrySet()) {
            List<Transcation> trans1 = new LinkedList<>();
            List<Transcation> trans2 = new LinkedList<>();
            for(Transcation trans : entry.getValue()) {
                if(trans.getDirection() == Direction.IN_BOUND) {
                    trans1.add(trans);
                }
                else trans2.add(trans);
            }
            if(!trans1.isEmpty()) part1.put(entry.getKey(), trans1);
            if(!trans2.isEmpty()) part2.put(entry.getKey(), trans2);
        }
        List<Map<AccountId, List<Transcation>>> result = new LinkedList<>();
        result.add(part1);
        result.add(part2);
        return result;
    }
    public Map<String, Money> sumAccountMoney(Map<String, List<Money>> accountMap) {
        Map<String, Money> result = new HashMap<>();
        for(Map.Entry<String, List<Money>> entry: accountMap.entrySet()) {
            BigDecimal total = BigDecimal.ZERO;
            for(Money money : entry.getValue()) {
                total = total.add(money.getAmount());
            }
            result.put(entry.getKey(), new Money(total, "USD"));
        }
        return result;
    }
    public static void main(String[] args) {
        List<Transcation> trans1 = new LinkedList<>();
        trans1.add(new Transcation(new AccountId(1L), Direction.IN_BOUND, new Money(new BigDecimal(5.0), "USD")));
        trans1.add(new Transcation(new AccountId(1L), Direction.OUT_BOUND, new Money(new BigDecimal(1.0), "USD")));
        trans1.add(new Transcation(new AccountId(1L), Direction.IN_BOUND, new Money(new BigDecimal(1.0), "USD")));
        trans1.add(new Transcation(new AccountId(1L), Direction.OUT_BOUND, new Money(new BigDecimal(4.0), "USD")));
        trans1.add(new Transcation(new AccountId(2L), Direction.IN_BOUND, new Money(new BigDecimal(5.0), "USD")));
        trans1.add(new Transcation(new AccountId(2L), Direction.OUT_BOUND, new Money(new BigDecimal(1.0), "USD")));
        trans1.add(new Transcation(new AccountId(2L), Direction.IN_BOUND, new Money(new BigDecimal(1.0), "USD")));
        trans1.add(new Transcation(new AccountId(2L), Direction.OUT_BOUND, new Money(new BigDecimal(3.0), "USD")));
        JavaCode checkBalance = new JavaCode();
        Map<AccountId, List<Transcation>> accountTrans = checkBalance.groupByAccountID(trans1);
        List<Map<AccountId, List<Transcation>>> partitioned = checkBalance.partitionInOut(accountTrans);
        System.out.println(partitioned);
    }
}
